from dataclasses import dataclass
from typing import Type

from sqlalchemy import asc, desc, func, or_
from sqlalchemy.orm import Query, Session
from trailblazer.constants import Pipeline
from trailblazer.dto.analysis_request import AnalysisRequest

from trailblazer.store.database import get_session
from trailblazer.store.filters.analyses_filters import (
    AnalysisFilter,
    apply_analysis_filter,
)
from trailblazer.store.models import Analysis, Job, Model


@dataclass
class BaseHandler:
    """All models in one base class."""

    def get_query(self, table: Type[Model]) -> Query:
        """Return a query for the given table."""
        session: Session = get_session()
        return session.query(table)

    def get_job_query_with_name_and_count_labels(self) -> Query:
        """Return a Job query with a name label and a count with a label."""
        session: Session = get_session()
        return session.query(
            Job.name.label("name"),
            func.count(Job.id).label("count"),
        )

    def get_analyses_query_by_search(self, analyses: Query, search_term: str) -> Query:
        """Return analyses by search term."""
        if search_term:
            analyses: Query = apply_analysis_filter(
                filter_functions=[AnalysisFilter.FILTER_BY_SEARCH_TERM],
                analyses=analyses,
                search_term=search_term,
            )
        return analyses

    def get_analyses_query_by_pipeline(self, pipeline: str) -> Query:
        """Return analyses by pipeline."""
        analyses: Query = self.get_query(Analysis)
        # Group existing variants of balsamic
        balsamic_pipeline: str = Pipeline.BALSAMIC.lower()
        if pipeline == balsamic_pipeline:
            analyses = analyses.filter(Analysis.data_analysis.startswith(balsamic_pipeline))
        if pipeline:
            analyses = analyses.filter(Analysis.data_analysis == pipeline)
        return analyses

    def get_filtered_analyses(self, analyses: Query, query: AnalysisRequest) -> Query:
        filters: list[AnalysisFilter] = []
        if query.status:
            filters.append(AnalysisFilter.FILTER_BY_STATUSES)
        if query.priority:
            filters.append(AnalysisFilter.FILTER_BY_PRIORITIES)
        if query.type:
            filters.append(AnalysisFilter.FILTER_BY_TYPES)
        if query.comment:
            filters.append(AnalysisFilter.FILTER_BY_EMPTY_COMMENT)
        return apply_analysis_filter(
            filter_functions=filters,
            analyses=analyses,
            comment=query.comment,
            statuses=query.status,
            priorities=query.priority,
            types=query.type,
        )

    def get_visible_analyses(self, analyses: Query) -> Query:
        return apply_analysis_filter(
            filter_functions=[AnalysisFilter.FILTER_BY_IS_VISIBLE],
            analyses=analyses,
        )

    def sort_analyses(self, analyses: Query, query: AnalysisRequest) -> Query:
        if query.sort_field:
            column = getattr(Analysis, query.sort_field)
            if query.sort_order == "asc":
                analyses = analyses.order_by(asc(column))
            else:
                analyses = analyses.order_by(desc(column))
        return analyses

    def paginate_analyses(self, analyses: Query, query: AnalysisRequest) -> Query:
        return analyses.limit(query.page_size).offset((query.page - 1) * query.page_size)

    def get_filtered_sorted_paginated_analyses(
        self, query: AnalysisRequest
    ) -> tuple[list[Analysis], int]:
        analyses: Query = self.get_analyses_query_by_pipeline(query.pipeline)
        analyses = self.get_filtered_analyses(analyses=analyses, query=query)
        analyses = self.get_analyses_query_by_search(analyses=analyses, search_term=query.search)
        analyses = self.sort_analyses(analyses=analyses, query=query)

        if not query.search:
            analyses = self.get_visible_analyses(analyses)

        total_analyses_count: int = analyses.count()
        query_page: Query = self.paginate_analyses(analyses=analyses, query=query)
        return query_page.all(), total_analyses_count



from datetime import datetime
from enum import Enum
from typing import Callable

import sqlalchemy
from sqlalchemy.orm import Query

from trailblazer.constants import TrailblazerStatus
from trailblazer.store.models import Analysis


def filter_analyses_by_comment(analyses: Query, comment: str, **kwargs) -> Query:
    """Filter analyses that contain the string given in 'comment'."""
    return analyses.filter(Analysis.comment.ilike(f"%{comment}%"))


def filter_analyses_by_case_id(analyses: Query, case_id: str, **kwargs) -> Query:
    """Filter analyses by case id."""
    return analyses.filter(Analysis.case_id == case_id)


def filter_analyses_by_entry_id(analyses: Query, analysis_id: int, **kwargs) -> Query:
    """Filter analyses by database entry id."""
    return analyses.filter(Analysis.id == analysis_id)


def filter_analyses_by_is_visible(analyses: Query, **kwargs) -> Query:
    """Filter analyses by case when is visible is true."""
    return analyses.filter(Analysis.is_visible.is_(True))


def filter_analyses_by_search_term(analyses: Query, search_term: str, **kwargs) -> Query:
    """Filter analyses by search term using multiple fields."""
    return analyses.filter(
        sqlalchemy.or_(
            Analysis.case_id.ilike(f"%{search_term}%"),
            Analysis.status.ilike(f"%{search_term}%"),
            Analysis.data_analysis.ilike(f"%{search_term}%"),
            Analysis.comment.ilike(f"%{search_term}%"),
        )
    )


def filter_analyses_by_started_at(analyses: Query, started_at: datetime, **kwargs) -> Query:
    """Filter analyses by when it started."""
    return analyses.filter(Analysis.started_at == started_at)


def filter_analyses_by_before_started_at(analyses: Query, started_at: datetime, **kwargs) -> Query:
    """Filter analyses by ifK it started before the supplied date."""
    return analyses.filter(Analysis.started_at < started_at)


def filter_analyses_by_status(analyses: Query, status: TrailblazerStatus, **kwargs) -> Query:
    """Filter analyses by status."""
    return analyses.filter(Analysis.status == status)


def filter_analyses_by_statuses(analyses: Query, statuses: list[str], **kwargs) -> Query:
    """Filter analyses by statuses."""
    return analyses.filter(Analysis.status.in_(statuses))


def filter_analyses_by_priorites(analyses: Query, priorities: list[str], **kwargs) -> Query:
    """Filter analyses by priorities."""
    return analyses.filter(Analysis.priority.in_(priorities))


def filter_analyses_by_types(analyses: Query, types: list[str], **kwargs) -> Query:
    """Filter analyses by types."""
    return analyses.filter(Analysis.type.in_(types))


def filter_analyses_by_empty_comment(analyses: Query, **kwargs) -> Query:
    return analyses.filter(sqlalchemy.or_(Analysis.comment.is_(None), Analysis.comment == ""))


class AnalysisFilter(Enum):
    """Define Analysis filter functions."""

    FILTER_BY_BEFORE_STARTED_AT: Callable = filter_analyses_by_before_started_at
    FILTER_BY_CASE_ID: Callable = filter_analyses_by_case_id
    FILTER_BY_COMMENT: Callable = filter_analyses_by_comment
    FILTER_BY_EMPTY_COMMENT: Callable = filter_analyses_by_empty_comment
    FILTER_BY_ENTRY_ID: Callable = filter_analyses_by_entry_id
    FILTER_BY_SEARCH_TERM: Callable = filter_analyses_by_search_term
    FILTER_BY_IS_VISIBLE: Callable = filter_analyses_by_is_visible
    FILTER_BY_PRIORITIES: Callable = filter_analyses_by_priorites
    FILTER_BY_STARTED_AT: Callable = filter_analyses_by_started_at
    FILTER_BY_STATUS: Callable = filter_analyses_by_status
    FILTER_BY_STATUSES: Callable = filter_analyses_by_statuses
    FILTER_BY_TYPES: Callable = filter_analyses_by_types


def apply_analysis_filter(
    analyses: Query,
    filter_functions: list[Callable],
    analysis_id: int | None = None,
    case_id: str | None = None,
    search_term: str | None = None,
    comment: str | None = None,
    started_at: datetime | None = None,
    status: str | None = None,
    statuses: list[str] | None = None,
    priorities: list[str] | None = None,
    types: list[str] | None = None,
) -> Query:
    """Apply filtering functions and return filtered results."""
    if statuses is None:
        statuses = []
    for function in filter_functions:
        analyses: Query = function(
            analyses=analyses,
            analysis_id=analysis_id,
            case_id=case_id,
            comment=comment,
            search_term=search_term,
            started_at=started_at,
            status=status,
            statuses=statuses,
            priorities=priorities,
            types=types,
        )
    return analyses



import datetime
import multiprocessing
import os
from http import HTTPStatus
from typing import Mapping

from flask import Blueprint, Response, abort, current_app, g, jsonify, make_response, request
from google.auth import jwt
from pydantic import ValidationError

from trailblazer.constants import (
    ONE_MONTH_IN_DAYS,
    TRAILBLAZER_TIME_STAMP,
    TrailblazerStatus,
)
from trailblazer.dto.analysis_request import AnalysisRequest
from trailblazer.dto.analysis_response import AnalysisResponse
from trailblazer.server.ext import store
from trailblazer.server.schemas import AnalysisUpdateRequest
from trailblazer.server.utils import parse_analysis_request
from trailblazer.services.analysis_service import AnalysisService
from trailblazer.store.models import Analysis, Info
from trailblazer.utils.datetime import get_date_number_of_days_ago

ANALYSIS_HOST: str = os.environ.get("ANALYSIS_HOST")

blueprint = Blueprint("api", __name__, url_prefix="/api/v1")


def stringify_timestamps(data: dict) -> dict[str, str]:
    """Convert datetime into string before dumping in order to avoid information loss"""
    for key, val in data.items():
        if isinstance(val, datetime.datetime):
            data[key] = str(val)
    return data


@blueprint.before_request
def before_request():
    """Authentication that is run before processing requests to the application"""
    if request.method == "OPTIONS":
        return make_response(jsonify(ok=True), 204)
    if os.environ.get("SCOPE") == "DEVELOPMENT":
        return
    if auth_header := request.headers.get("Authorization"):
        jwt_token = auth_header.split("Bearer ")[-1]
    else:
        return abort(403, "no JWT token found on request")

    user_data: Mapping = jwt.decode(jwt_token, verify=False)
    if user := store.get_user(email=user_data["email"], exclude_archived=True):
        g.current_user = user
    else:
        return abort(403, f"{user_data['email']} doesn't have access")


@blueprint.route("/analyses")
def analyses():
    """Display analyses."""
    analysis_service: AnalysisService = current_app.extensions.get("analysis_service")
    try:
        query: AnalysisRequest = parse_analysis_request(request)
        response: AnalysisResponse = analysis_service.get_analyses(query)
        return jsonify(response.model_dump()), HTTPStatus.OK
    except ValidationError as error:
        return jsonify(error=str(error)), HTTPStatus.BAD_REQUEST


@blueprint.route("/analyses/<int:analysis_id>", methods=["GET", "PUT"])
def analysis(analysis_id):
    """Retrieve or update an analysis."""
    analysis: Analysis = store.get_analysis_with_id(analysis_id)
    if analysis is None:
        return abort(404)

    if request.method == "PUT":
        try:
            analysis_update = AnalysisUpdateRequest.model_validate(request.json)
            analysis = store.update_analysis(
                analysis_id=analysis_id,
                comment=analysis_update.comment,
                status=analysis_update.status,
                is_visible=analysis_update.is_visible,
            )
        except ValidationError as error:
            return jsonify(error=str(error)), HTTPStatus.BAD_REQUEST

    data = analysis.to_dict()
    data["jobs"] = [job.to_dict() for job in analysis.jobs]
    data["user"] = analysis.user.to_dict() if analysis.user else None
    return jsonify(**data)


@blueprint.route("/info")
def info():
    """Display metadata about database."""
    info: Info = store.get_query(table=Info).first()
    return jsonify(**info.to_dict())


@blueprint.route("/me")
def me():
    """Return information about a logged in user."""
    return jsonify(**g.current_user.to_dict())


@blueprint.route("/aggregate/jobs")
def aggregate_jobs():
    """Return stats about failed jobs."""
    number_of_days_ago = int(request.args.get("days_back", ONE_MONTH_IN_DAYS))
    time_window: datetime = get_date_number_of_days_ago(number_of_days_ago)
    failed_jobs: list[dict[str, str | int]] = store.get_nr_jobs_with_status_per_category(
        status=TrailblazerStatus.FAILED, since_when=time_window
    )
    return jsonify(jobs=failed_jobs)


@blueprint.route("/update-all")
def update_analyses():
    """Update all ongoing analysis by querying SLURM."""
    process = multiprocessing.Process(
        target=store.update_ongoing_analyses,
        kwargs={"analysis_host": ANALYSIS_HOST},
    )
    process.start()
    return jsonify(f"Success! Trailblazer updated {datetime.datetime.now()}"), HTTPStatus.CREATED


@blueprint.route("/update/<int:analysis_id>", methods=["PUT"])
def update_analysis(analysis_id):
    """Update a specific analysis."""
    try:
        process = multiprocessing.Process(
            target=store.update_run_status,
            kwargs={"analysis_id": analysis_id, "analysis_host": ANALYSIS_HOST},
        )
        process.start()
        return jsonify("Success! Update request sent"), HTTPStatus.CREATED
    except Exception as error:
        return jsonify(f"Exception: {error}"), HTTPStatus.CONFLICT


@blueprint.route("/cancel/<int:analysis_id>", methods=["PUT"])
def cancel(analysis_id):
    """Cancel an analysis and all slurm jobs associated with it."""
    auth_header = request.headers.get("Authorization")
    jwt_token = auth_header.split("Bearer ")[-1]
    user_data = jwt.decode(jwt_token, verify=False)
    try:
        process = multiprocessing.Process(
            target=store.cancel_ongoing_analysis,
            kwargs={
                "analysis_id": analysis_id,
                "analysis_host": ANALYSIS_HOST,
                "email": user_data["email"],
            },
        )
        process.start()
        return jsonify("Success! Cancel request sent"), HTTPStatus.CREATED
    except Exception as error:
        return jsonify(f"Exception: {error}"), HTTPStatus.CONFLICT


@blueprint.route("/delete/<int:analysis_id>", methods=["PUT"])
def delete(analysis_id):
    """Delete an analysis and all slurm jobs associated with it."""
    try:
        process = multiprocessing.Process(
            target=store.delete_analysis,
            kwargs={"analysis_id": analysis_id, "force": True},
        )
        process.start()
        return jsonify("Success! Delete request sent!"), HTTPStatus.CREATED
    except Exception as error:
        return jsonify(f"Exception: {error}"), HTTPStatus.CONFLICT


# CG REST INTERFACE ###
# ONLY POST routes which accept messages in specific format
# NOT for use with GUI (for now)


@blueprint.route("/get-latest-analysis", methods=["POST"])
def post_get_latest_analysis():
    """Return latest analysis entry for specified case id."""
    post_request = request.json
    case_id: str = post_request.get("case_id")
    if latest_case_analysis := store.get_latest_analysis_for_case(case_id):
        raw_analysis: dict[str, str] = stringify_timestamps(latest_case_analysis.to_dict())
        return jsonify(**raw_analysis), HTTPStatus.OK
    return jsonify(None), HTTPStatus.OK


@blueprint.route("/find-analysis", methods=["POST"])
def post_find_analysis():
    """Find analysis using case id, date, and status."""
    post_request: Response.json = request.json
    case_id: str = post_request.get("case_id")
    started_at = datetime.strptime(post_request.get("started_at"), TRAILBLAZER_TIME_STAMP).date()
    status: str = post_request.get("status")
    if analysis := store.get_analysis(
        case_id=case_id,
        started_at=started_at,
        status=status,
    ):
        raw_analysis: dict[str, str] = stringify_timestamps(analysis.to_dict())
        return jsonify(**raw_analysis), HTTPStatus.OK
    return jsonify(None), HTTPStatus.OK


@blueprint.route("/delete-analysis", methods=["POST"])
def post_delete_analysis():
    """Delete analysis using analysis_id. If analysis is ongoing, an error will be raised.
    To delete ongoing analysis, --force flag should also be passed.
    If an ongoing analysis is deleted in ths manner, all ongoing jobs will be cancelled"""
    post_request: Response.json = request.json
    analysis_id: str = post_request.get("analysis_id")
    force: str = post_request.get("force")
    try:
        store.delete_analysis(analysis_id=analysis_id, force=force)
        return jsonify(None), HTTPStatus.CREATED
    except Exception as error:
        return jsonify(f"Exception: {error}"), HTTPStatus.CONFLICT


@blueprint.route("/mark-analyses-deleted", methods=["POST"])
def post_mark_analyses_deleted():
    """Mark all analysis belonging to a case as deleted."""
    post_request: Response.json = request.json
    case_id: str = post_request.get("case_id")
    case_analyses: list[Analysis] | None = store.update_case_analyses_as_deleted(case_id)
    if raw_analysis := [
        stringify_timestamps(case_analysis.to_dict()) for case_analysis in case_analyses
    ]:
        return jsonify(*raw_analysis), HTTPStatus.CREATED
    return jsonify(None), HTTPStatus.CREATED


@blueprint.route("/add-pending-analysis", methods=["POST"])
def post_add_pending_analysis():
    """Add new analysis with status: pending."""
    post_request: Response.json = request.json
    try:
        analysis: Analysis = store.add_pending_analysis(
            case_id=post_request.get("case_id"),
            email=post_request.get("email"),
            type=post_request.get("type"),
            config_path=post_request.get("config_path"),
            out_dir=post_request.get("out_dir"),
            priority=post_request.get("priority"),
            data_analysis=post_request.get("data_analysis"),
            ticket_id=post_request.get("ticket"),
            workflow_manager=post_request.get("workflow_manager"),
        )
        raw_analysis: dict = stringify_timestamps(analysis.to_dict())
        return jsonify(**raw_analysis), 201
    except Exception as exception:
        return jsonify(f"Exception: {exception}"), 409


@blueprint.route("/set-analysis-uploaded", methods=["PUT"])
def set_analysis_uploaded():
    """Set the analysis uploaded at attribute."""
    put_request: Response.json = request.json
    try:
        store.update_analysis_uploaded_at(
            case_id=put_request.get("case_id"), uploaded_at=put_request.get("uploaded_at")
        )
        return jsonify("Success! Uploaded at request sent"), HTTPStatus.CREATED
    except Exception as error:
        return jsonify(f"Exception: {error}"), HTTPStatus.CONFLICT


@blueprint.route("/set-analysis-status", methods=["PUT"])
def set_analysis_status():
    """Update analysis status of a case with supplied status."""
    put_request: Response.json = request.json
    try:
        case_id: str = put_request.get("case_id")
        status: str = put_request.get("status")
        store.update_analysis_status(case_id=case_id, status=status)
        return (
            jsonify(f"Success! Analysis set to {put_request.get('status')} request sent"),
            HTTPStatus.CREATED,
        )
    except Exception as error:
        return jsonify(f"Exception: {error}"), HTTPStatus.CONFLICT


@blueprint.route("/add-comment", methods=["PUT"])
def add_comment():
    """Updating comment on analysis."""
    put_request: Response.json = request.json
    try:
        case_id: str = put_request.get("case_id")
        comment: str = put_request.get("comment")
        store.update_analysis_comment(case_id=case_id, comment=comment)
        return jsonify("Success! Adding comment request sent"), HTTPStatus.CREATED
    except Exception as error:
        return jsonify(f"Exception: {error}"), HTTPStatus.CONFLICT



import logging

from sqlalchemy.orm import Session

from trailblazer.constants import TrailblazerStatus
from trailblazer.exc import MissingAnalysis, TrailblazerError
from trailblazer.store.base import BaseHandler
from trailblazer.store.database import get_session
from trailblazer.store.models import Analysis

LOG = logging.getLogger(__name__)


class DeleteHandler(BaseHandler):
    """Class for deleting items in the database."""

    def delete_analysis_jobs(self, analysis: Analysis) -> None:
        """Delete all jobs linked to the given analysis."""
        session: Session = get_session()
        for job in analysis.jobs:
            session.delete(job)
        session.commit()

    def delete_analysis(self, analysis_id: int, force: bool = False) -> None:
        """Delete the analysis from the database. Also delete ongoing analysis if 'force' is True.
        Raises:
            MissingAnalysis when no analysis.
            TrailblazerError for ongoing analysis for analysis id.
        """
        session: Session = get_session()
        analysis: Analysis | None = self.get_analysis_with_id(analysis_id=analysis_id)
        if not analysis:
            raise MissingAnalysis("Analysis not found")
        if not force and analysis.status in TrailblazerStatus.ongoing_statuses():
            raise TrailblazerError(
                f"Analysis for {analysis.case_id} is currently ongoing! Use --force flag to delete ongoing analysis."
            )
        LOG.info(f"Deleting analysis: {analysis_id}, for case: {analysis.case_id}")
        session.delete(analysis)
        session.commit()



from trailblazer.io.csv import read_csv_stream


def test_get_content_from_stream(csv_stream: str):
    """Tests read CSV stream."""
    # GIVEN a string in CSV format

    # WHEN reading the CSV content in string
    raw_csv_content: list[list[str]] = read_csv_stream(stream=csv_stream)

    # THEN assert a list is returned
    assert isinstance(raw_csv_content, list)

    # THEN the content should match the expected content
    expected_content = [
        ["Lorem", "ipsum", "sit", "amet"],
        ["value_1", "value_2", "value_3", "value_4"],
    ]
    assert raw_csv_content == expected_content


def test_get_content_from_stream_to_dict(csv_stream: str):
    """Tests read CSV stream into a dict."""
    # GIVEN a string in CSV format

    # WHEN reading the CSV content in string
    raw_csv_content: list[list[str]] = read_csv_stream(read_to_dict=True, stream=csv_stream)

    # Then assert a list is returned and that the first element is a dict
    assert isinstance(raw_csv_content, list)
    assert isinstance(raw_csv_content[0], dict)



from pydantic import BaseModel, Field

from trailblazer.constants import TrailblazerPriority, TrailblazerStatus, TrailblazerTypes


class AnalysisRequest(BaseModel):
    pipeline: str | None = ""
    search: str | None = None
    page_size: int | None = Field(alias="pageSize", default=250)
    page: int | None = 1
    sort_field: str | None = Field(alias="sortField", default="started_at")
    sort_order: str | None = Field(alias="sortOrder", default="desc")
    status: list[TrailblazerStatus] | None = []
    priority: list[TrailblazerPriority] | None = []
    type: list[TrailblazerTypes] | None = []
    comment: list[str] | None = []



from datetime import datetime
from enum import Enum
from typing import Callable

from sqlalchemy.orm import Query

from trailblazer.store.models import Job


def filter_jobs_by_since_when(jobs: Query, since_when: datetime, **kwargs) -> Query:
    """Filter jobs before the supplied date."""
    return jobs.filter(Job.started_at > since_when)


def filter_jobs_by_status(jobs: Query, status: str, **kwargs) -> Query:
    """Filter jobs with matching status."""
    return jobs.filter(Job.status == status)


def filter_jobs_by_analysis_id(jobs: Query, analysis_id: str, **kwargs) -> Query:
    """Filter jobs with matching analysis id."""
    return jobs.filter(Job.analysis_id == analysis_id)


def sort_jobs_by_started_at(jobs: Query, **kwargs) -> Query:
    """Sort jobs by start date."""
    return jobs.order_by(Job.started_at.desc())


class JobFilter(Enum):
    """Define Job filter functions."""

    FILTER_BY_SINCE_WHEN: Callable = filter_jobs_by_since_when
    FILTER_BY_STATUS: Callable = filter_jobs_by_status
    FILTER_BY_ANALYSIS_ID: Callable = filter_jobs_by_analysis_id
    SORT_BY_STARTED_AT: Callable = sort_jobs_by_started_at


def apply_job_filter(
    jobs: Query,
    filter_functions: list[Callable],
    since_when: datetime | None = None,
    status: str | None = None,
    analysis_id: str | None = None,
) -> Query:
    """Apply filtering functions and return filtered results."""
    for function in filter_functions:
        jobs: Query = function(
            jobs=jobs,
            since_when=since_when,
            status=status,
            analysis_id=analysis_id,
        )
    return jobs



from json import JSONDecodeError
from pathlib import Path

from trailblazer.apps.tower.api import TowerAPI
from trailblazer.apps.tower.models import TowerTaskResponse, TowerWorkflowResponse
from trailblazer.constants import FileFormat
from trailblazer.io.controller import ReadFile


class MockTowerAPI(TowerAPI):
    """Instance of TowerAPI that mimics expected Tower output."""

    @property
    def response(self) -> dict | None:
        return self.mock_response or None

    @property
    def tasks_response(self) -> dict | None:
        return self.mock_tasks_response or None

    def mock_query(self, response_file: Path) -> TowerWorkflowResponse:
        try:
            self.mock_response = TowerWorkflowResponse(
                **ReadFile.get_content_from_file(
                    file_format=FileFormat.JSON, file_path=response_file
                )
            )
        except JSONDecodeError:
            self.mock_response = TowerWorkflowResponse(**{})
        return self.mock_response

    def mock_tasks_query(self, response_file: Path) -> TowerTaskResponse:
        try:
            self.mock_tasks_response = TowerTaskResponse(
                **ReadFile.get_content_from_file(
                    file_format=FileFormat.JSON, file_path=response_file
                )
            )
        except JSONDecodeError:
            self.mock_tasks_response = TowerTaskResponse(**{})
        return self.mock_tasks_response



from flask import Request

from trailblazer.dto.analysis_request import AnalysisRequest


def parse_analysis_request(request: Request) -> AnalysisRequest:
    """Parse a request for retrieving analyses."""
    query_params = {}
    for key in request.args.keys():
        if key.endswith("[]"):
            query_params[key[:-2]] = request.args.getlist(key)
        else:
            query_params[key] = request.args.get(key)
    return AnalysisRequest.model_validate(query_params)



import json
from flask.testing import FlaskClient
from http import HTTPStatus

from trailblazer.constants import TrailblazerStatus
from trailblazer.server.schemas import AnalysisUpdateRequest
from trailblazer.store.models import Analysis

TYPE_JSON = "application/json"


def test_update_analysis_status(client: FlaskClient, analysis: Analysis):
    # GIVEN an analysis with a pending status

    # GIVEN a valid request to set the status to completed
    request = AnalysisUpdateRequest(status=TrailblazerStatus.COMPLETED)
    data: str = request.model_dump_json()

    # WHEN updating the analysis to be completed
    response = client.put(f"/api/v1/analyses/{analysis.id}", data=data, content_type=TYPE_JSON)

    # THEN it gives a success response
    assert response.status_code == HTTPStatus.OK

    # THEN it should return the analysis
    assert response.json["id"] == analysis.id

    # THEN it should have the new status
    assert response.json["status"] == TrailblazerStatus.COMPLETED


def test_update_analysis_comment(client: FlaskClient, analysis: Analysis):
    # GIVEN an analysis

    # GIVEN a valid request to set the comment
    new_comment = "new comment"
    request = AnalysisUpdateRequest(comment=new_comment)
    data: str = request.model_dump_json()

    # WHEN updating the analysis with a comment
    response = client.put(f"/api/v1/analyses/{analysis.id}", data=data, content_type=TYPE_JSON)

    # THEN it gives a success response
    assert response.status_code == HTTPStatus.OK

    # THEN it returns the analysis
    assert response.json["id"] == analysis.id

    # THEN it has the new comment
    assert response.json["comment"] == new_comment


def test_update_analysis_visibility(client: FlaskClient, analysis: Analysis):
    # GIVEN an analysis tagged as visible

    # GIVEN a valid request to set the visibility
    request = AnalysisUpdateRequest(is_visible=False)
    data: str = request.model_dump_json()

    # WHEN updating the analysis with a comment
    response = client.put(f"/api/v1/analyses/{analysis.id}", data=data, content_type=TYPE_JSON)

    # THEN it gives a success response
    assert response.status_code == HTTPStatus.OK

    # THEN it returns the analysis
    assert response.json["id"] == analysis.id

    # THEN it has the new visibility
    assert response.json["is_visible"] == False


def test_update_analysis_invalid_request(client: FlaskClient, analysis: Analysis):
    # GIVEN an invalid request
    data: str = json.dumps({"status": "invalid_status"})

    # WHEN updating the analysis with an invalid request
    response = client.put(f"/api/v1/analyses/{analysis.id}", data=data, content_type=TYPE_JSON)

    # THEN it gives a bad request response
    assert response.status_code == HTTPStatus.BAD_REQUEST


def test_delete_comment_on_analysis(client: FlaskClient, analysis: Analysis):
    # GIVEN a valid request to delete a comment
    data: str = json.dumps({"comment": ""})

    # WHEN sending the request
    response = client.put(f"/api/v1/analyses/{analysis.id}", data=data, content_type=TYPE_JSON)

    # THEN it gives a success response
    assert response.status_code == HTTPStatus.OK

    # THEN it returns the analysis without a comment
    assert response.json["comment"] == ""
